using System;
using System.Data;
using System.Xml;
using System.IO;

namespace XMLChapterCS {

	public class CDataSet : DataSet {
		public string GetXml(string[] cdataSections) {
			return InsertCDATASections(cdataSections);
		}

		private string InsertCDATASections(string[] cdataSections) {
			//Convert to XML with expanded general entities and CDATA sections
			//as appropriate
			XmlValidatingReader reader = null;
			XmlTextWriter writer = null;
			StringWriter sw = null;
			Array.Sort(cdataSections);
			try {
				reader = new XmlValidatingReader(this.GetXml(),XmlNodeType.Document,null);
				sw = new StringWriter();
				writer = new XmlTextWriter(sw);
				writer.Formatting = Formatting.Indented;
				reader.ValidationType = ValidationType.None;
				reader.EntityHandling = EntityHandling.ExpandCharEntities;
				string currentElement = String.Empty;
				while (reader.Read()) {
					switch (reader.NodeType) {
						case XmlNodeType.Element:
							currentElement = reader.Name;
							writer.WriteStartElement(currentElement);
							writer.WriteAttributes(reader,false);
							break;
						case XmlNodeType.Text:
							if (Array.BinarySearch(cdataSections,currentElement) < 0) {
								writer.WriteString(reader.Value);
							} else {
								writer.WriteCData(reader.Value);
							}
							break;                    
						case XmlNodeType.EndElement:
							writer.WriteEndElement();
							break;
						default:
							break;
					}
				}
			}
			catch (Exception exp) {
				return exp.Message;
			}
			finally {
				reader.Close();
				writer.Close();
			}
			return sw.ToString();
		}
	}
}
