using System;
using System.Web;
using System.Web.Caching;
using System.Configuration;
using System.Xml;
using System.Xml.Serialization;
using XMLChapterCS.Configuration;

namespace XMLChapterCS {

	public class ConfigFileReader {

		public static string GetConnectionString(string server) {
			ServerConfig config = GetConfig();
			//Check if Web server matches with dev server name or not
			Array.Sort(config.SqlServer.Dev.Servers);
			if (Array.BinarySearch(config.SqlServer.Dev.Servers,server) > -1) {
				return config.SqlServer.Dev.ConnString;
			}

			//Check if Web server matches with prod server name or not
			Array.Sort(config.SqlServer.Prod.Servers);
			if (Array.BinarySearch(config.SqlServer.Prod.Servers,server) > -1) {
				return config.SqlServer.Prod.ConnString;
			}
			
			//Default is null
			return null;
		}

		public static ServerConfig GetConfig() {
			HttpContext context = HttpContext.Current;
			//Check if config is already loaded into cache
			if (context.Cache.Get("ServerConfig") == null) {
				XmlTextReader reader = null;
				try {
					string configPath = context.Server.MapPath(ConfigurationSettings.AppSettings["ServerConfig"]);				
					reader = new XmlTextReader(configPath);
					//Deserialize XML configuration data
					XmlSerializer s = new XmlSerializer(typeof(ServerConfig));
					ServerConfig config = (ServerConfig)s.Deserialize(reader);
					//Create file dependency for cache
					CacheDependency cd = new CacheDependency(new string[]{configPath});
					context.Cache.Insert("ServerConfig",config,cd);
					return config;
				}
				catch {
					throw new ApplicationException("Unable to find config file.");
				}
				finally {
					if (reader != null) reader.Close();
				}
			} else {
				return (ServerConfig)context.Cache.Get("ServerConfig");
			}
		}
	}

}
