using System;
using System.Web;
using System.Web.Caching;
using System.Xml.XPath;
using System.Configuration;

namespace XMLChapterCS {

	public class ConfigReader {
		public static string GetConfigValue(string xpath) {
			XPathDocument doc = GetConfigDocument();
			XPathNavigator nav = doc.CreateNavigator();
			nav.MoveToRoot();
			nav.MoveToFirstChild();
			XPathNodeIterator it = nav.Select(xpath);
			if (it != null) {
				it.MoveNext();
				return it.Current.Value;
			} else {
				return null;
			}
		}

		public static string GetConnectionString(string server) {
			return GetConfigValue("SqlServer/" + GetServerType(server).ToString() + 
				"/@ConnString");
		}

		public static ServerType GetServerType(string server) {
			string currServer = server.ToLower();
			XPathDocument doc = GetConfigDocument();
			XPathNavigator nav = doc.CreateNavigator();
			nav.MoveToRoot();
			nav.MoveToFirstChild();
			string xpath = "//Servers/Server";
			XPathNodeIterator it = nav.Select(xpath);
			while (it.MoveNext()) {
				//Check if server is in production
				if (it.Current.Value.ToLower() == currServer) {
					it.Current.MoveToParent();  //Move to ServerNames
					it.Current.MoveToParent();  //Move to server type
                    return (ServerType)Enum.Parse(typeof(ServerType),
						it.Current.Name,true);
				}
			}
			//Default to dev server
			return ServerType.Dev;
		}

		private static XPathDocument GetConfigDocument() {
			HttpContext context = HttpContext.Current;
			//Check if config is already loaded into cache
			if (context.Cache.Get("ServerConfig") == null) {
				try {
					string configPath = context.Server.MapPath(ConfigurationSettings.AppSettings["ServerConfig"]);				
					XPathDocument doc = new XPathDocument(configPath);
					//Create file dependency for cache
					CacheDependency cd = new CacheDependency(new string[]{configPath});
					//Cache XPathDocument instance
					context.Cache.Insert("ServerConfig",doc,cd);
					return doc;
				}
				catch {
					throw new ApplicationException("Unable to find config file.");
				}
			} else {
				//Return XPathDocument already in cache
				return (XPathDocument)context.Cache.Get("ServerConfig");
			}
		}
	}

	public enum ServerType {
		Prod,
		Dev
	}
}
