Imports System
Imports System.Web
Imports System.Web.Caching
Imports System.Configuration
Imports System.Xml
Imports System.Xml.Serialization
Imports XMLChapterVB.Configuration

Public Class ConfigFileReader

  Public Shared Function GetConnectionString(ByVal server As String) As String
    Dim config As ServerConfig = GetConfig()
    'Check if Web server matches with dev server name or not
    Array.Sort(config.SqlServer.Dev.Servers)
    If Array.BinarySearch(config.SqlServer.Dev.Servers, server) > -1 Then
      Return config.SqlServer.Dev.ConnString
    End If

    'Check if Web server matches with prod server name or not
    Array.Sort(config.SqlServer.Prod.Servers)
    If Array.BinarySearch(config.SqlServer.Prod.Servers, server) > -1 Then
      Return config.SqlServer.Prod.ConnString
    End If

    'Default is null
    Return Nothing
  End Function


  Public Shared Function GetConfig() As ServerConfig
    Dim context As HttpContext = HttpContext.Current
    'Check if config is already loaded into cache
    If context.Cache.Get("ServerConfig") Is Nothing Then
      Dim reader As XmlTextReader = Nothing
      Try
        Dim configPath As String = context.Server.MapPath(ConfigurationSettings.AppSettings("ServerConfig"))
        reader = New XmlTextReader(configPath)
        'Deserialize XML configuration data
        Dim s As New XmlSerializer(GetType(ServerConfig))
        Dim config As ServerConfig = CType(s.Deserialize(reader), ServerConfig)
        'Create file dependency for cache
        Dim cd As New CacheDependency(New String() {configPath})
        context.Cache.Insert("ServerConfig", config, cd)
        Return config
      Catch
        Throw New ApplicationException("Unable to find config file.")
      Finally
        If Not (reader Is Nothing) Then
          reader.Close()
        End If
      End Try
    Else
      Return CType(context.Cache.Get("ServerConfig"), ServerConfig)
    End If
  End Function

End Class

