<%@Page Language="VB" Debug="True" %>
<%@Import Namespace="System.Data" %>
<%@Import Namespace="System.Data.OleDb" %>

<%-------------------------------------------------------------%>

<script runat="server">

Sub Page_Load()

  If Not Page.IsPostback Then
    BindDataGrid()
  End If

End Sub

<%-------------------------------------------------------------%>

Sub BindDataGrid()

  ' declare SQL statement to fill table
  Dim sProducts As String _
    = "SELECT ProductID, ProductName, UnitPrice, Discontinued " _
    & "FROM Products " _
    & "WHERE ProductID <= 6 ORDER BY ProductName"

  Dim sConnect As String _
    = ConfigurationSettings.AppSettings("NorthwindOleDbConnectString")

  Dim oConnect As New OleDbConnection(sConnect)

  ' create new DataSet and fill from database
  Dim oDS As New DataSet()

  Try

    Dim oDA As New OleDbDataAdapter(sProducts, oConnect)
    oDA.Fill(oDS, "Products")

  Catch oErr As Exception

    ' be sure to close connection if error occurs
    If oConnect.State <> ConnectionState.Closed  Then
      oConnect.Close()
    End If

    ' display error message in page
    lblErr.Text &= oErr.Message & "<br />"

  End Try

  ' bind DataGrid to Products table
  dgr1.DataSource = oDS
  dgr1.DataMember = "Products"
  dgr1.DataBind()

End Sub

<%-------------------------------------------------------------%>

Sub BindRowData(sender As Object, e As DataGridItemEventArgs)

  ' see what type of row (header, footer, item, etc.) caused the event
  Dim oType As ListItemType = CType(e.Item.ItemType, ListItemType)

  If oType = ListItemType.Item Or oType = ListItemType.AlternatingItem Then

    ' add primary key of current row to each control in row
    ' required to perform multiple-row updates after a postback
    ' hide it in the style attribute as "rowval:key"
    Dim oTextbox As Textbox = CType(e.Item.FindControl("txtProductName"), Textbox)
    oTextbox.Style("rowval") = e.Item.DataItem("ProductID")

    oTextbox = CType(e.Item.FindControl("txtPrice"), Textbox)
    oTextbox.Style("rowval") = e.Item.DataItem("ProductID")

    Dim oCheckbox As Checkbox = CType(e.Item.FindControl("chkDiscontinued"), Checkbox)
    oCheckbox.Style("rowval") = e.Item.DataItem("ProductID")

  End If

End Sub

<%-------------------------------------------------------------%>

Function DoItemUpdate(sRowKey As String, _
                      sColumnName As String, _
                      sValue As String) As Boolean

  ' create a suitable SQL statement and execute it
  Dim sSQL As String
  sSQL = "UPDATE Products SET " & sColumnName & "=" _
       & sValue & " WHERE ProductID=" & sRowKey
  lblErr.Text &= "Executing SQL statement: " & sSQL & "<br />"

  Dim sConnect As String _
    = ConfigurationSettings.AppSettings("NorthwindOleDbConnectString")
  Dim oConnect As New OleDbConnection(sConnect)

  Try

    oConnect.Open()
    Dim oCommand As New OleDbCommand(sSQL, oConnect)
    If oCommand.ExecuteNonQuery() <> 1 Then
      lblErr.Text &= "ERROR: Could not update the selected row<br />"
      Return False
    Else
      Return True
    End If
    oConnect.Close()

  Catch oErr As Exception

    ' be sure to close connection if error occurs
    If oConnect.State <> ConnectionState.Closed  Then
      oConnect.Close()
    End If

    ' display error message in page
    lblErr.Text &= "ERROR: " & oErr.Message & "<br />"
    Return False

  End Try

End Function

<%-------------------------------------------------------------%>

Sub ProductNameChanged(sender As Object, e As EventArgs)

  ' remove any previous formatting
  sender.Style.Remove("color")

  ' execute update against data store
  ' DoItemUpdate returns false if update is not performed
  If Not DoItemUpdate(sender.Style("rowval"), "ProductName", _
         "'" & sender.Text.Replace("'", "''") & "'") Then

    ' change text to red
    sender.Style.Add("color", "Red")
    WriteClientScript(sender.UniqueID)

  End If

End Sub

<%-------------------------------------------------------------%>

Sub PriceChanged(sender As Object, e As EventArgs)

  sender.Style.Remove("color")
  If Not DoItemUpdate(sender.Style("rowval"), "UnitPrice", _
         sender.Text.Replace("'", "''")) Then
    sender.Style.Add("color", "Red")
    WriteClientScript(sender.UniqueID)
  End If

End Sub

<%-------------------------------------------------------------%>

Sub DiscontinuedChanged(sender As Object, e As EventArgs)

  ' remove any previous formatting
  sender.Style.Remove("background-color")

  ' get value of Checkbox as an integer (not True/False)
  Dim iChecked As Integer = (sender.Checked = True)

  '*************************************************
  ' for demonstration only, cause an update error if
  ' ProductID is 6 by applying invalid value
  Dim sForceError As String = ""
  If sender.Style("rowval") = "6" Then
    sForceError = "*NaN*"
  End If
  '*************************************************

  If Not DoItemUpdate(sender.Style("rowval"), "Discontinued", _
         iChecked.ToString() & sForceError) Then

    ' change background to red
    sender.Style.Add("background-color", "Red")
    WriteClientScript(sender.UniqueID)

  End If

End Sub

<%-------------------------------------------------------------%>

Sub WriteClientScript(sCtrlID As String)

  ' create script to set focus to first control with error
  ' see if previous instance of this control has already
  ' added the required JavaScript code to the page
  If Not Page.IsClientScriptBlockRegistered("AHHGridFocus") Then

    Dim sScript As String = vbCrlf _
    & "<script language='javascript'>" & vbCrlf _
    & "<!--" & VbCrlf _
    & "var ctrl = document.getElementById('" & sCtrlID & "');" & vbCrlf _
    & "ctrl.focus();" & vbCrlf _
    & "ctrl.select();" & vbCrlf _
    & "// -->" & vbCrlf _
    & "<" & "/script>" & vbCrlf

    ' add this JavaScript code to the page
    Page.RegisterStartupScript("AHHGridFocus", sScript)

  End If

  ' make Cancel button visible. Will disappear again on
  ' next postback because EnableViewState in False
  btnRefresh.Visible = True

End Sub

<%-------------------------------------------------------------%>

Sub DoRefresh(sender As Object, e As EventArgs)

  ' refill grid from original data source
  BindDataGrid()

End Sub

</script>

<%-------------------------------------------------------------%>
<%-------------------------------------------------------------%>

<html>
<head>
<!-- #include file="../global/style.inc" -->
<title>Updating Multiple Rows Using 'Changed' Events</title>
</head>
<body>

<span class="heading">Updating Multiple Rows Using 'Changed' Events</span><hr />

<form runat="server">

  <asp:DataGrid id="dgr1" runat="server"
       Font-Size="10" Font-Name="Tahoma,Arial,Helvetica,sans-serif"
       BorderStyle="None" BorderWidth="1px" BorderColor="#deba84"
       BackColor="#DEBA84" CellPadding="5" CellSpacing="1"
       OnItemDataBound="BindRowData"
       AutoGenerateColumns="False">
    <HeaderStyle Font-Bold="True" ForeColor="#ffffff" BackColor="#b50055" />
    <ItemStyle BackColor="#FFF7E7" VerticalAlign="Top" />
    <AlternatingItemStyle backcolor="#ffffc0" />

    <Columns>

      <asp:TemplateColumn HeaderText="ID"
           HeaderStyle-HorizontalAlign="Center"
           ItemStyle-HorizontalAlign="Center">
        <ItemTemplate>
          <%# Container.DataItem("ProductID") %>
        </ItemTemplate>
      </asp:TemplateColumn>

      <asp:TemplateColumn HeaderText="Product"
           HeaderStyle-HorizontalAlign="Center"
           ItemStyle-HorizontalAlign="Right">
        <ItemTemplate>
          <asp:Textbox id="txtProductName" Columns="40"
               Text='<%# Container.DataItem("ProductName") %>'
               OnTextChanged="ProductNameChanged"
               runat="server" />
        </ItemTemplate>
      </asp:TemplateColumn>

      <asp:TemplateColumn HeaderText="Price"
           HeaderStyle-HorizontalAlign="Center"
           ItemStyle-HorizontalAlign="Right">
        <ItemTemplate>
          <asp:Textbox id="txtPrice" Columns="6"
               Text='<%# DataBinder.Eval(Container.DataItem, _
                         "UnitPrice", "${0:F2}") %>'
               OnTextChanged="PriceChanged"
               runat="server" />
        </ItemTemplate>
      </asp:TemplateColumn>

      <asp:TemplateColumn HeaderText="Discontinued"
           ItemStyle-HorizontalAlign="Center">
        <ItemTemplate>
          <asp:Checkbox id="chkDiscontinued"
               Checked='<%# Container.DataItem("Discontinued") %>'
               OnCheckedChanged="DiscontinuedChanged"
               runat="server" />
        </ItemTemplate>
      </asp:TemplateColumn>

    </Columns>
  </asp:DataGrid>

<p class="cite">To create an update error: use a Product name more than 40 characters long, put a<br />
non-numeric value in the Price column, or change the Discontinued checkbox for the<br />
row with ID value of 6. Also remember to edit the connection strings in your web.config<br />
file for the Northwind sample database.</p>

<asp:Button Text="Update" id="btnUpdate" runat="server" /> &nbsp;
<asp:Button Text="Cancel" id="btnRefresh" Visible="False"
            OnClick="DoRefresh" EnableViewState="False" runat="server" /><p />
<asp:Label id="lblErr" EnableViewState="False" runat="server" />

</form>

<hr />
<!-- #include file="../global/viewsource.inc" -->
<!-- #include file="../global/footer.inc" -->

</body>
</html>

