<%@Page Language="VB" Debug="True" %>
<%@Import Namespace="System.Data" %>
<%@Import Namespace="System.Data.OleDb" %>
<%@Import Namespace="System.Data.SqlClient" %>
<%@Import Namespace="System.Data.Odbc" %>

<script runat="server">

<%-------------------------------------------------------------%>

Dim bLocal As Boolean   ' true if running on local machine

Sub Page_Load()

  ' check if running on local server
  Dim sPath As String = Request.Url.ToString()
  bLocal = (sPath.IndexOf("http://localhost/") = 0)

  ' execute queries for each connection type
  lblSql.Text &= ExecuteCommand("Sql")
  lblOleDb.Text &= ExecuteCommand("OleDb")
  lblJetOleDb.Text &= ExecuteCommand("JetOleDb")
  lblOdbc.Text &= ExecuteCommand("Odbc")
  lblJetOdbc.Text &= ExecuteCommand("JetOdbc")

End Sub

<%-------------------------------------------------------------%>

Function ExecuteCommand(sClassName As String) As String

  Dim sCon, sResult As String
  Dim oCon, oCmd, oParam

  ' select appropriate objects depending on namespace in use
  Select Case sClassName
    Case "Sql":
      sCon = ConfigurationSettings.AppSettings("NorthwindSqlClientConnectString")
      oCon = New SqlConnection(sCon)
      oCmd = New SqlCommand("ParamOrderProc", oCon)
    Case "OleDb":
      sCon = ConfigurationSettings.AppSettings("NorthwindOleDbConnectString")
      oCon = New OleDbConnection(sCon)
      oCmd = New OleDbCommand("ParamOrderProc", oCon)
    Case "JetOleDb":
      sCon = ConfigurationSettings.AppSettings("NorthwindJetOleDbConnectString") _
           & Request.MapPath("../databases/jetdb.mdb")
      oCon = New OleDbConnection(sCon)
      oCmd = New OleDbCommand("ParamOrderProc", oCon)
    Case "Odbc":
      sCon = ConfigurationSettings.AppSettings("NorthwindOdbcConnectString")
      oCon = New OdbcConnection(sCon)
      oCmd = New OdbcCommand("{call ParamOrderProc(?, ?, ?)}", oCon)
    Case "JetOdbc":
      sCon = ConfigurationSettings.AppSettings("NorthwindJetOdbcConnectString") _
           & Request.MapPath("../databases/jetdb.mdb")
      oCon = New OdbcConnection(sCon)
      oCmd = New OdbcCommand("{call ParamOrderProc(?, ?, ?)}", oCon)
  End Select

  ' only show connection string if running on local server
  If bLocal Then
    sResult = "Connection String: " & sCon & "<br/>"
  End If

  Try

    ' specify query type, add parameters and execute query
    oCmd.CommandType = CommandType.StoredProcedure
    oCmd.Parameters.Add("@Param1", "Value1")
    oCmd.Parameters.Add("@Param3", "Value3")
    oCmd.Parameters.Add("@Param2", "Value2")
    oCon.Open()
    sResult &= "Result: <b>" & oCmd.ExecuteScalar().ToString() & "</b>"
    oCon.Close()
    Return sResult

  Catch oErr As Exception

    ' be sure to close connection if error occurs
    If oCon.State <> ConnectionState.Closed  Then
      oCon.Close()
    End If

    sResult &= "ERROR: " & oErr.Message
    Return sResult
  End Try


End Function

</script>

<%-------------------------------------------------------------%>

<html>
<head>
<!-- #include file="../global/style.inc" -->
<title>Stored Procedure and Query Parameter Ordering</title>
</head>
<body>


<span class="heading">Stored Procedure and Query Parameter Ordering</span><hr />

<div>
Executing the SQL Server stored procedure:<pre>
CREATE PROCEDURE ParamOrderProc
@Param1 varchar (10) = 'Default1',
@Default varchar (10) = 'Default2',
@Param2 varchar (10) = 'Default3',
@Param3 varchar (10) = 'Default4' AS
SELECT @Param1 + ', ' + @Default + ', ' + @Param2 + ', ' + @Param3</pre>
and the Access Query:<pre>
PARAMETERS Param1 Text(10), Param2 Text(10), Param3 Text(10);
SELECT [Param1] + ', ' + [Param2] + ', ' + [Param3] AS Expr1;</pre>
but supplying the parameters as:<pre>
oCmd.Parameters.Add("@Param1", "Value1")
oCmd.Parameters.Add("@Param3", "Value3")
oCmd.Parameters.Add("@Param2", "Value2")</pre>
</div>
<p />

<asp:Label Text="<b>Using the SqlClient classes with SQL Server</b><br />"
           id="lblSql" EnableViewState="False" runat="server" /><p />
<asp:Label Text="<b>Using the OleDb classes with SQL Server</b><br />"
           id="lblOleDb" EnableViewState="False" runat="server" /><p />
<asp:Label Text="<b>Using the OleDb classes with the JET provider</b><br />"
           id="lblJetOleDb" EnableViewState="False" runat="server" /><p />
<asp:Label Text="<b>Using the Odbc classes with SQL Server</b><br />"
           id="lblOdbc" EnableViewState="False" runat="server" /><p />
<asp:Label  Text="<b>Using the Odbc classes with the JET driver</b><br />"
           id="lblJetOdbc" EnableViewState="False" runat="server" /><p />

<span class="cite">
Remember to edit the connection strings in web.config file for the Northwind
sample database, and<br />install the ParamOrderProc stored procedure using the SQL
script provided in the databases folder.
</span>
<hr />
<!-- #include file="../global/viewsource.inc" -->
<!-- #include file="../global/footer.inc" -->

</body>
</html>

