Imports System.ComponentModel
Imports System.Web.UI
Imports System.Web.UI.HtmlControls
Imports System.Configuration

Namespace SAMS.PageTemplates

    Public Class MasterPage
        Inherits System.Web.UI.Page

        Private Const ErrOnlyContent As String = "Content page can only contain <sams:Content /> controls: {0}"
        Private Const ErrNoMaster As String = "MasterPageFile property not set"
        Private Const ErrNoHolder As String = "<sams:ContentPlaceHolder id='{0}'/> must be defined in {1}"

        Public Event MasterInit As System.EventHandler

        Private _contents As New ArrayList
        Private _template As Control
        Private _masterPageFile As String = String.Empty

        Public Property MasterPageFile() As String
            Get
                Return _masterPageFile
            End Get
            Set(ByVal Value As String)
                _masterPageFile = Value
            End Set
        End Property

        Protected Overrides Sub OnInit(ByVal e As EventArgs)

            MyBase.OnInit(e)

            ' raise the event to allow the user to set the master page
            RaiseEvent MasterInit(Me, e)

            Me.BuildMasterPage()
            Me.BuildContents()

        End Sub

        Protected Overrides Sub AddParsedSubObject(ByVal obj As Object)

            If TypeOf (obj) Is Content Then
                ' add it to the internal controls collection
                _contents.Add(obj)
            Else
                ' Should only allow controls of type Content, but we should allow blank 
                ' literal content, so the master page can have whitespace for easy reading
                If TypeOf (obj) Is LiteralControl Then
                    Dim ctl As LiteralControl = CType(obj, LiteralControl)
                    If ctl.Text.Trim <> "" Then
                        Throw New Exception(String.Format(ErrOnlyContent, ctl.Text))
                    End If
                ElseIf Not (TypeOf (obj) Is LiteralControl) Then
                    Throw New Exception(String.Format(ErrOnlyContent, obj.ToString))
                End If
            End If

        End Sub

        ' add the controls from the master file
        Private Sub BuildMasterPage()

            If _masterPageFile = String.Empty Then
                ' if not set at the page level check for set at at the config level
                _masterPageFile = ConfigurationSettings.AppSettings("MasterPageFile")
                If _masterPageFile = String.Empty Then
                    Throw New Exception(ErrNoMaster)
                End If
            End If

            ' load the master file
            Me._template = Me.Page.LoadControl(Me._masterPageFile)

            ' iterate through the controls of the master file, adding
            ' them to the internal controls collection
            Dim index As Integer
            For index = 0 To Me._template.Controls.Count - 1
                Dim ctl As Control = Me._template.Controls(0)
                Me._template.Controls.Remove(ctl)
                If (ctl.Visible) Then
                    Me.Controls.Add(ctl)
                End If
            Next
            Me.Controls.AddAt(0, Me._template)

        End Sub

        ' add the controls from the content page
        Private Sub BuildContents()

            Dim ct As Content
            For Each ct In Me._contents
                Dim holder As Control = Me.FindControl(ct.ID)

                ' control with same name must be of type ContentPlaceHolder
                If holder Is Nothing Or Not (TypeOf (holder) Is ContentPlaceHolder) Then
                    Throw New Exception(String.Format(ErrNoHolder, ct.ID, _masterPageFile))
                End If

                ' only clear default content if the Content control actually has content
                If ct.HasControls Then
                    holder.Controls.Clear()
                End If

                ' add the individual controls from the current page
                Dim index As Integer
                For index = 0 To ct.Controls.Count - 1
                    holder.Controls.Add(ct.Controls(0))
                Next
            Next

        End Sub
    End Class

    Public Class ContentPlaceHolder
        Inherits System.Web.UI.WebControls.Panel

        Public Sub New()
        End Sub

        Public Overrides Sub RenderBeginTag(ByVal writer As HtmlTextWriter)
        End Sub

        Public Overrides Sub RenderEndTag(ByVal writer As HtmlTextWriter)
        End Sub
    End Class

    Public Class Content
        Inherits System.Web.UI.WebControls.Panel

        Public Sub New()
        End Sub

        Public Overrides Sub RenderBeginTag(ByVal writer As HtmlTextWriter)
        End Sub

        Public Overrides Sub RenderEndTag(ByVal writer As HtmlTextWriter)
        End Sub
    End Class

End Namespace