Imports System
Imports System.ComponentModel
Imports System.Web.UI
Imports System.Web.UI.WebControls

Namespace SAMS.PageTemplates

    Public Class MasterPageControlDynamic
        Inherits System.Web.UI.WebControls.WebControl
        Implements INamingContainer

        Protected _menuCell As TableCell
        Protected _contentCell As TableCell
        Protected _headerContent As UserControl
        Protected _menuContent As UserControl
        Protected _footerContent As UserControl

#Region "   Templates"
        ' The template used for the menu
        Dim _menuTemplate As ITemplate
        '<TemplateContainer(GetType(TemplateControl))> _
        Public Property MenuTemplate() As ITemplate
            Get
                Return _menuTemplate
            End Get
            Set(ByVal Value As ITemplate)
                _menuTemplate = Value
            End Set
        End Property

        ' The template used for the content
        Dim _contentTemplate As ITemplate
        '<TemplateContainer(GetType(TemplateControl))> _
        Public Property ContentTemplate() As ITemplate
            Get
                Return _contentTemplate
            End Get
            Set(ByVal Value As ITemplate)
                _contentTemplate = Value
            End Set
        End Property
#End Region

        Protected Overrides Sub CreateChildControls()

            Controls.Clear()

            ' load the default content
            _headerContent = Page.LoadControl("Templates\HeaderTemplate.ascx")
            _footerContent = Page.LoadControl("Templates\FooterTemplate.ascx")
            Me.Controls.Add(_headerContent)
            Me.Controls.Add(_footerContent)

            ' create the table cell for the menu and
            ' instantiate the controls within the template
            _menuCell = New TableCell
            If Not (_menuTemplate Is Nothing) Then
                _menuTemplate.InstantiateIn(_menuCell)
            End If
            Me.Controls.Add(_menuCell)

            ' create the table cell for the content and
            ' instantiate the controls within the template
            _contentCell = New TableCell
            If Not (_contentTemplate Is Nothing) Then
                _contentTemplate.InstantiateIn(_contentCell)
            End If
            Me.Controls.Add(_contentCell)

        End Sub

        Protected Overrides Sub Render(ByVal writer As System.Web.UI.HtmlTextWriter)

            writer.AddAttribute(HtmlTextWriterAttribute.Width, "100%")
            writer.AddAttribute(HtmlTextWriterAttribute.Border, "1")
            writer.RenderBeginTag(HtmlTextWriterTag.Table)

            ' header 
            writer.RenderBeginTag(HtmlTextWriterTag.Tr)
            writer.AddAttribute(HtmlTextWriterAttribute.Colspan, "2")
            writer.RenderBeginTag(HtmlTextWriterTag.Td)
            _headerContent.RenderControl(writer)
            writer.RenderEndTag() ' td
            writer.RenderEndTag() ' tr

            ' menu
            writer.RenderBeginTag(HtmlTextWriterTag.Tr)
            writer.AddAttribute(HtmlTextWriterAttribute.Valign, "top")
            writer.AddAttribute(HtmlTextWriterAttribute.Width, "15%")
            If _menuTemplate Is Nothing _
                OrElse _menuCell.HasControls = False _
                And _menuCell.Text.Trim = "" Then
                ' either the template hasn't been supplied, or it has been
                ' supplied but with no contained controls. So add default content.
                _menuContent = Page.LoadControl("Templates\MenuTemplate.ascx")
                _menuCell.Controls.Add(_menuContent)
            End If
            _menuCell.RenderControl(writer)

            ' content
            writer.AddAttribute(HtmlTextWriterAttribute.Valign, "top")
            _contentCell.RenderControl(writer)
            writer.RenderEndTag() ' tr

            ' footer
            writer.RenderBeginTag(HtmlTextWriterTag.Tr)
            writer.AddAttribute(HtmlTextWriterAttribute.Colspan, "2")
            writer.RenderBeginTag(HtmlTextWriterTag.Td)
            _footerContent.RenderControl(writer)
            writer.RenderEndTag() ' td
            writer.RenderEndTag() ' tr

            writer.RenderEndTag() ' table

        End Sub

    End Class

End Namespace